<?php

namespace domain\User\User;

use domain\Communication\Email\Email;
use domain\User\UserProducer\UserProducerBuilder;
use domain\User\UserProducer\UserProducerRepository;
use domain\_\AbstractService;
use domain\_\ManagerInterface;

class NewsletterManager extends AbstractService implements ManagerInterface
{
    protected UserProducerRepository $userProducerRepository;
    protected UserProducerBuilder $userProducerBuilder;

    public function loadDependencies(): void
    {
        $this->userProducerRepository = $this->loadService(UserProducerRepository::class);
        $this->userProducerBuilder = $this->loadService(UserProducerBuilder::class);
    }

    public function isUserSubscribedNewsletter(User $user = null, string $type = Email::TYPE_NEWSLETTER): bool
    {
        if($user) {
            $userProducer = $this->userProducerRepository->findOneUserProducer($user);
            $fieldNewsletter = ($type == Email::TYPE_NEWSLETTER) ? 'newsletter' : 'newsletter_order_taking';
            if($userProducer && $userProducer->$fieldNewsletter) {
                return true;
            }
        }

        return false;
    }

    public function isUserSubscribedOneNewsletter(User $user = null): bool
    {
        if(!$user) {
            return false;
        }

        return $this->isUserSubscribedNewsletter($user, Email::TYPE_NEWSLETTER)
            || $this->isUserSubscribedNewsletter($user, Email::TYPE_ORDER_TAKING);
    }

    public function manageUserNewsletter(User $user, ?bool $newsletter, ?bool $newsletterOrderTaking)
    {
        if($newsletter) {
            $this->subscribeUserNewsletter($user, Email::TYPE_NEWSLETTER);
        }
        else {
            $this->unsubscribeUserNewsletter($user, Email::TYPE_NEWSLETTER);
        }

        if($newsletterOrderTaking) {
            $this->subscribeUserNewsletter($user, Email::TYPE_ORDER_TAKING);
        }
        else {
            $this->unsubscribeUserNewsletter($user, Email::TYPE_ORDER_TAKING);
        }
    }

    public function subscribeUserNewsletter(User $user, string $type): bool
    {
        $userProducer = $this->userProducerBuilder->createUserProducerIfNotExist($user, $this->getProducerContext());
        return $this->userProducerBuilder->updateUserProducerNewsletterByType(
            $userProducer,
            true,
            $type
        );
    }

    public function unsubscribeUserNewsletter(User $user, string $type): bool
    {
        $userProducer = $this->userProducerRepository->findOneUserProducer($user);
        if($userProducer) {
            return $this->userProducerBuilder->updateUserProducerNewsletterByType(
                $userProducer,
                false,
                $type
            );
        }

        return false;
    }
}